import binascii

from .util import CRYPTO_LOOKUP

vendor_lookup = {
    0: 'Synapse',
    2: 'Freescale',
    3: 'CEL',
    4: 'ATMEL',
    5: 'Silicon Labs',
    7: 'PC',
    9: 'STMicrosystems',
}

interface_lookup = {
    0: '802.15.4-based 2.4 GHz',
    1: 'None (serial only)',
    3: '868 MHz',
    4: 'Powerline',
    5: '900 MHz Frequency-Hopping',
    6: '802.15.4-based 900 MHz',
}

cpu_lookup = {
    0: 'Freescale MC9S08GT60A',
    1: 'ZIC 8051',
    2: 'MC9S08QE',
    3: 'Coldfire',
    4: 'ARM7',
    5: 'ATmega',
    6: 'Si100x 8051',
    7: 'X86',
    8: 'UNKNOWN',
    10: 'ARM CORTEX M3',
}

platform_lookup = {
    0: 'Synapse RF100 SNAP Engine',
    3: 'CEL ZIC2410',
    5: 'MC1321x',
    6: 'ATmega128RFA1',
    7: 'SNAPcom',
    8: 'Si100x',
    9: 'MC1322x',
    11: 'Si100x KADEX',
    13: 'Synapse RF300 SNAP Engine',
    14: 'Synapse RF200 SNAP Engine',
    15: 'Synapse SM300 Surface Mount Module',
    16: 'Synapse SM301 Surface Mount Module',
    17: 'Synapse SM200 Surface Mount Module',
    19: 'Synapse RF266',
    20: 'STM32W108xB',
    27: 'Synapse SM220 Surface Mount Module',
}

build_lookup = {
    0: 'Debug',
    1: 'Release',
}


def passthrough_none(original):
    """This decorator wraps a formatter function, causing it to return None if
    the input value is None and to call the original formatter function
    otherwise. This prevents formatters from having to explicitly handle None inputs.
    """

    return lambda value: None if value is None else original(value)


@passthrough_none
def format_noop(value):
    return value


@passthrough_none
def format_as_boolean(value):
    return bool(value)


@passthrough_none
def format_as_unsigned(value):
    return value & 0xffff


@passthrough_none
def format_as_hex(value):
    return "0x%04x" % format_as_unsigned(value)


@passthrough_none
def format_as_mac(value):
    return ':'.join([binascii.hexlify(byte) for byte in value])


@passthrough_none
def format_as_encryption_status(value):
    return CRYPTO_LOOKUP.get(value, value)


@passthrough_none
def format_as_vendor(value):
    return vendor_lookup.get(value, value)


@passthrough_none
def format_as_interface(value):
    return interface_lookup.get(value, value)


@passthrough_none
def format_as_cpu(value):
    return cpu_lookup.get(value, value)


@passthrough_none
def format_as_platform(value):
    return platform_lookup.get(value, value)


@passthrough_none
def format_as_build(value):
    return build_lookup.get(value, value)
