"""SNAPstraction low level ops"""
from tornado.gen import coroutine, Return

from .node import fix_up_target


@coroutine
@fix_up_target
def callback_ucast_rpc(gus, target, func, *args, **kwargs):
    """Callback Unicast RPC"""
    data = {}
    val = yield gus.callback_ucast_rpc(target, func, *args, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def callback_mcast_rpc(gus, addrs, func, *args, **kwargs):
    """Callback Multicast RPC"""
    data = {}
    val = yield gus.callback_mcast_rpc(addrs, func, *args, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def callback_dmcast_rpc(gus, addrs, func, *args, **kwargs):
    """Callback Directed Multicast RPC"""
    data = {}
    val = yield gus.callback_dmcast_rpc(addrs, func, *args, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def callback_ucast_data(gus, target, payload, **kwargs):
    """Callback Unicast Data"""
    data = {}
    val = yield gus.callback_ucast_data(target, payload, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def callback_mcast_data(gus, targets, payload, **kwargs):
    """Callback Multicast Data"""
    data = {}
    val = yield gus.callback_mcast_data(targets, payload, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def ucast_rpc(gus, target, func, *args):
    """Unicast RPC"""
    data = {}
    val = yield gus.ucast_rpc(target, func, *args)
    data['response'] = val
    raise Return(data)


@coroutine
def mcast_rpc(gus, func, *args, **kwargs):
    """Multicast RPC"""
    data = {}
    val = yield gus.mcast_rpc(func, *args, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def dmcast_rpc(gus, targets, func, *args, **kwargs):
    """Directed Multicast RPC"""
    data = {}
    val = yield gus.dmcast_rpc(targets, func, *args, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
@fix_up_target
def ucast_data(gus, target, payload, **kwargs):
    """Unicast Data"""
    data = {}
    val = yield gus.ucast_data(target, payload, **kwargs)
    data['response'] = val
    raise Return(data)


@coroutine
def mcast_data(gus, payload, **kwargs):
    """Multicast Data"""
    data = {}
    val = yield gus.mcast_data(payload, **kwargs)
    data['response'] = val
    raise Return(data)
